#ifndef _STUDENTCOLLECTION_CPP
#define _STUDENTCOLLECTION_CPP

#include <string>
#include <fstream>

#include "MyTmpltLib.h"

#include "StudentCollection.h"

using namespace std;

StudentCollection::StudentCollection()
{
   try
   {
      scReserved = scIncr;
      scArr = new Student[ scReserved ];
      scNOfStudents = 0; 
   }
   catch( bad_alloc& )
   {
      throw StudentCollectionXptn( "StudentCollection",
                                  StudentCollectionXptn::allocFailed);
   }
   scPAllEnrollments = 0;
} // StudentCollection::StudentCollection

void StudentCollection::erase1Student( int ndx )
{
   scArr[ndx] = scArr[scNOfStudents-1];
   --scNOfStudents;
} // StudentCollection::erase1Student

const Student& StudentCollection::get1Student( int aIdNum ) const
{
   int ndx( findNdx(aIdNum) );
   if ( ndx < 0 )
      throw StudentCollectionXptn( "get1Student",
                                  StudentCollectionXptn::notFound,
                                   aIdNum );
   return scArr[ndx];
} // StudentCollection::get1Student

void StudentCollection::add1Course( int aIdNum,
                                    const Course& aCourse )
{
   int ndx( findNdx(aIdNum) );
   if ( ndx < 0 )
      throw StudentCollectionXptn( "add1Course",
                                  StudentCollectionXptn::notFound,
                                   aIdNum );
   scArr[ndx].add1Course( aCourse );
} // StudentCollection::add1Course

void StudentCollection::delete1Course( int aIdNum,
                                       const Course& aCourse )
{
   int ndx( findNdx(aIdNum) );
   if ( ndx < 0 )
      throw StudentCollectionXptn( "delete1Course",
                                  StudentCollectionXptn::notFound,
                                   aIdNum );
   scArr[ndx].delete1Course( aCourse );
} // StudentCollection::add1Course

void StudentCollection::save( ofstream& bout, SIndexEntry* index )
{
   if ( bout.fail() )
      throw StudentCollectionXptn( "save",
                             StudentCollectionXptn::fileNotOpen );
   bout.write( reinterpret_cast<const char*>(&scNOfStudents),
               sizeof(scNOfStudents) );
   for ( int k(0); k < scNOfStudents; ++k )
   {
      index[k].sIdNum = scArr[k].getIdNum();
      index[k].loc = bout.tellp();
      scArr[k].save( bout );
   }
   if ( bout.fail() )
      throw StudentCollectionXptn( "save",
                             StudentCollectionXptn::cannotWrite );
} // StudentCollection::save

void StudentCollection::load( ifstream& bin )
{
   StudentCollection tmp;
   unsigned int n;
   bin.read( reinterpret_cast<char*>(&n), sizeof(scNOfStudents) );
   if ( !bin.eof() )
   {
      for ( int k(0); k < n && !bin.fail(); ++k )
      {
         Student oneStudent;
         oneStudent.load( bin );
         tmp.insert1Student( oneStudent );
      }
      if ( bin.fail() )
         throw StudentCollectionXptn( "load",
                              StudentCollectionXptn::cannotRead );
      swapArr( tmp );
   }
} // StudentCollection::load

void StudentCollection::swapArr( StudentCollection& rhs )
{
   std::swap( scArr, rhs.scArr );
   std::swap( scNOfStudents, rhs.scNOfStudents );
   std::swap( scReserved, rhs.scReserved ); 
} // StudentCollection::swapArr

void StudentCollection::display( ostream& tout ) const
{
   tout << scNOfStudents << endl;
   for ( int k(0); k < scNOfStudents; ++k )
      scArr[k].display( tout );
}

int StudentCollection::findNdx( int aIdNum ) const
{
   int ndx;
   if ( aIdNum <= 0 )
      ndx = -1;
   else
      ndx = linSearch( scArr, scNOfStudents,
                       0, scNOfStudents-1, Student(aIdNum) );
   return ndx;
} // StudentCollection::findNdx

void StudentCollection::insert1Student( const Student& aStudent )
{
   if ( scReserved <= scNOfStudents+1 )
   {
      try {  renew( scArr, scNOfStudents, scReserved+scIncr );
//           renew<Student, StudentXptn>( scArr, scNOfStudents, scReserved+scIncr );
             scReserved += scIncr;  }
      catch( MyTmpltLibXptn& )
      {
         throw StudentCollectionXptn( "insert1Student",
                                 StudentCollectionXptn::allocFailed );
      }
   }
   scArr[scNOfStudents] = aStudent;
   ++scNOfStudents; 
} // StudentCollection::insert1Student

void StudentCollection::add1Student( const Student& aStudent )
{
   int ndx( findNdx(aStudent.getIdNum()) );
   if ( ndx < 0 ) //    
   {
      if ( scPAllEnrollments == 0 )
         throw StudentCollectionXptn( "add1Student",
                                StudentCollectionXptn::noEnroll );
      insert1Student( aStudent );
      const Course::CourseKey*
                              aStCourses( aStudent.getCourses() );
      for ( int k(0); k < aStudent.getNoOfCourses(); ++k )
      {
         scPAllEnrollments->add1StudentInCourse(
          StudentInCourse(aStudent.getIdNum(), aStCourses[k].s) );
      }
   }
} // StudentCollection::add1Student

void StudentCollection::delete1Student( int aIdNum )
{
   int ndx( findNdx(aIdNum) );
   if ( ndx >= 0 )  // 
   {
      if ( scArr[ndx].getNoOfCourses() > 0 )
         throw StudentCollectionXptn( "delete1Student",
                                 StudentCollectionXptn::enrollRef,
                                      aIdNum );
      erase1Student( ndx );
   }
} // StudentCollection::delete1Student

void StudentCollection::checkWH( ostream& log, int maxWH ) const
{
   if ( maxWH <= 0 )
      throw StudentCollectionXptn( "checkWH",
                                   StudentCollectionXptn::negWH,
                                                          maxWH );
   for ( int k(0); k < scNOfStudents; ++k )
   {
      if ( scArr[k].getWH() > maxWH )
         log << "student with id num " << scArr[k].getIdNum()
             << ": " << scArr[k].getWH() << " hours/week" << endl;
   } // for
} // StudentCollection::checkWH

#endif // _STUDENTCOLLECTION_CPP

